""" Errors returned by the API """

_error_map = {
    "steam_profile_private": "The requested steam user profile is private.",
    "resource_already_exists": (
        "The selected driver is already connected to your company."
    ),
    "driver_no_compatible_games": (
        "The requested steam user doesn't own ETS2 or ATS "
        "(This could be due to it being private on their profile.)"
    ),
    "max_driver_limit_reached": (
        "You reached the maximum number of drivers for your account. "
        "Please upgrade to continue adding more drivers."
    ),
    "driver_does_not_exist": "Driver could not be found.",
}


class TrackSimError(Exception):
    def __init__(self, error):
        Exception.__init__(self, _error_map.get(error["error"]))



import requests
from typing import Union
from .errors import TrackSimError
from .types import Driver


class TrackSim:
    """TrackSim Class for the API"""

    def __init__(self, token: str, host: str = "https://api.tracksim.app/v1") -> None:
        """TrackSim Class to interfact with the

        Parameters:
            ``token`` (``str``):
                The Authentication Token from the Integrations Page
                https://tracksim.app/integrations

            ``host`` (``str``, optional):
                The API Host. Defaults to "https://api.tracksim.app/v1".

        """
        self._host = host
        self._session = requests.Session()
        self._session.headers.update(
            {"Content-Type": "application/json", "Authorization": f"Api-Key {token}"}
        )

    def _make_request(self, path: str, method: str = "get", **kwargs):
        """Make HTTP Requests against the API

        Parameters:
            ``path`` (``str``):
                The API calls' path for the request

            ``method`` (``str``, optional):
                HTTP Method (GET, POST, PUT, etc). Defaults to "GET".

            ``**kwargs``:
                Additional arguments passed to the API call.

        Returns:
            JSON serialized data

        """
        req = self._session.request(method, f"{self._host}/{path}", **kwargs)
        if req.status_code in [200, 201]:
            return req.json()
        else:
            raise TrackSimError(req.json())

    def add_driver(self, steam_id: Union[int, str]) -> Driver:
        """
        Adds a driver to the company.

        ---
        Parameters:
            ``steam_id`` (``int`` | ``str``):
                The drivers' SteamID64.

        ---
        Returns:
            ``Driver``: The Driver as a JSON object

        """
        req = self._make_request(
            path="drivers/add", method="POST", json={"steam_id": f"{steam_id}"}
        )
        return Driver(**req)

    def remove_driver(self, steam_id: Union[int, str]) -> None:
        """Removes a Driver from the company.

        ---
        Parameters:
            ``steam_id`` (``int`` | ``str``):
                The drivers' SteamID64.

        ---
        Returns:
            ``None``: On success, nothing is returned.

        """
        req = self._make_request(
            path="drivers/remove", method="DELETE", json={"steam_id": f"{steam_id}"}
        )
        return req

    def get_driver(self, steam_id: Union[int, str]) -> Driver:
        """Retrieves information about a Driver from the company.

        ---
        Parameters:
            ``steam_id`` (``str``):
                The drivers' SteamID64.

        Returns:
            ``Driver``: The driver as a JSON object

        """
        req = self._make_request(path=f"drivers/{steam_id}/details")
        return Driver(**req)

    def manage_driver(
        self,
        steam_id: Union[int, str],
        ets_job_logging: bool = True,
        ets_live_tracking: bool = True,
        ats_job_logging: bool = True,
        ats_live_tracking: bool = True,
    ) -> Driver:
        """Change settings about a driver.

        ---
        Parameters:
            ``steam_id`` (``str``):
                The drivers' SteamID64.

            ``ets_job_logging`` (``bool``, optional):
                Whether to log jobs for the driver. Defaults to True.

            ``ets_live_tracking`` (``bool``, optional):
                Whether to show the driver on the ETS2 Live Map. Defaults to True.

            ``ats_job_logging`` (``bool``, optional):
                Whether to log jobs for the driver in ATS. Defaults to True.

            ``ats_live_tracking`` (``bool``, optional):
                Whether tho show the driver on the ATS Live Map. Defaults to True.

        ---
        Returns:
            ``Driver``: The driver as a JSON object.

        """
        req = self._make_request(
            path=f"drivers/{steam_id}/manage",
            method="PATCH",
            json={
                "eut2_job_logging": ets_job_logging,
                "eut2_live_tracking": ets_live_tracking,
                "ats_job_logging": ats_job_logging,
                "ats_live_tracking": ats_live_tracking,
            },
        )
        return Driver(**req)

    def get_job_route(self, job_id: Union[int, str]) -> list:
        """Get the route details about as specific job.

        ---
        Parameters:
            ``job_id`` (``int`` | ``str``):
                The ID of the job

        ---
        Returns:
            ``list``:
                An array of ``dict``s containing X and Z coordinates and a UNIX Timestamp
        """
        req = self._make_request(
            path=f"jobs/{job_id}/route",
        )
        return req



import base64
import json


class List(list):
    """Custom List Class to beautify output"""

    __slots__ = []

    def __str__(self) -> None:
        return TrackSimType.__str__(self)

    def __repr__(self) -> None:
        return (
            f"tracksim.types.List([{','.join(TrackSimType.__repr__(i) for i in self)}])"
        )


class TrackSimType:
    def __init__(self) -> None:
        pass

    @staticmethod
    def default(obj: "TrackSimType"):
        if isinstance(obj, bytes):
            return repr(obj)

        return {
            "_": obj.__class__.__name__,
            **{
                attr: getattr(obj, attr)
                for attr in filter(lambda x: not x.startswith("_"), obj.__dict__)
                if getattr(obj, attr) is not None
            },
        }

    def __str__(self) -> str:
        return json.dumps(self, indent=4, default=TrackSimType.default)

    def __repr__(self) -> str:
        return f"<{self.__class__.__name__}: {self.__dict__}>"


class Driver(TrackSimType):
    id: int
    steam_id: str
    username: str
    profile_photo_url: str
    client: "Client"
    settings: "Settings"
    is_banned: bool
    last_active: str

    def __init__(
        self,
        id: int,
        steam_id: str,
        username: str,
        profile_photo_url: str,
        client: object,
        settings: object,
        is_banned: bool,
        last_active: str,
    ) -> None:
        self.id = id
        self.steam_id = steam_id
        self.username = (
            # ! Fix this shit lmao
            # This is such a mess because the API returns non-ASCII characters
            # as B64 encoded strings as they're stored like that in the DB.
            # * TrackSim devs are aware.
            base64.b64decode(username.replace("B64_", "")).decode()
            if username.startswith("B64_")
            else username
        )
        self.profile_photo_url = profile_photo_url
        self.client = Client(**client)
        self.settings = Settings(**settings)
        self.is_banned = is_banned
        self.last_active = last_active


class Client(TrackSimType):
    is_installed: bool
    version: "Version"

    def __init__(self, is_installed: bool, version: object) -> None:
        self.is_installed = is_installed
        self.version = Version(**version)


class Version(TrackSimType):
    version: str
    branch: str
    platform: str

    def __init__(self, version: str, branch: str, platform: str) -> None:
        self.version = version
        self.branch = branch
        self.platform = platform


class Settings(TrackSimType):
    eut2: "EUT2"
    ats: "ATS"

    def __init__(self, eut2: object, ats: object) -> None:
        self.eut2 = EUT2(**eut2)
        self.ats = ATS(**ats)


class EUT2(TrackSimType):
    job_logging: bool
    live_tracking: bool

    def __init__(self, job_logging: bool, live_tracking: bool) -> None:
        self.job_logging = job_logging
        self.live_tracking = live_tracking


class ATS(TrackSimType):
    job_logging: bool
    live_tracking: bool

    def __init__(self, job_logging: bool, live_tracking: bool) -> None:
        self.job_logging = job_logging
        self.live_tracking = live_tracking
